/* primos_subs.c
/* Copyright (c) 1988, Prime Computer, Inc., Natick, MA 01760
/*                     All Rights Reserved
*/

/* PRIMOS versions of subroutines called by IO benchmark.

   Initial Version: 12 August 1988 by Barry Wolman

          Modified: 29 August 1988 by Barry Wolman
              1. Fix setrwlock to work on path like <disk>file.
              2. Change run to use phntm$
              3. Add kill
              4. Set errno as appropriate

          Modified: 28 September 1988 by Barry Wolman
              1. Add time_of_day routine
              2. Make timdat structure generally available

          Modified:  4 April 1989 by Barry Wolman
              1. Add append flag to monitor_system

          Modified: 17 August 1990 by Barry Wolman
              1. Rename systimes to cptimes & add dummy fullcptimes
              2. Add chmod & getinfo
*/

#include <stdio.h>
#include <keys.h>
#include <errd.h>
#include "ioincl.h"

#define LOCKSEGNO  02037     /* Works on EN.C09;  check your system */

fortran void cp$(),phntm$(),logo$$();

/* Following defines the PL/I-style varying string needed to communicate
   with some PRIMOS gates */

struct cv {
    short     length;
    char      string[256];
    };

/* Following structure defines an IX-mode pointer */

struct ptr {
    int       junk:4;
    int       segno:12;
    int       offset:16;
    };

/* Allocate 128KB of shared memory for process user */

int *get_shared_memory(user)
int user;
    {
    int * p;
    register struct ptr * q;

    p = NULL;
    q = (struct ptr *)&p;
    q -> segno = LOCKSEGNO;
    q -> offset = 0;
    return(p);
    }

/* Free shared memory */

free_shared_memory(user)
int user;
    {
    }

/* Can't understand why there's no unlink subroutine in Prime's library */

unlink(name)
char *name;
    {
    return(delete(name));
    }

/* PRIMOS anaogue of getenv.  Note: global variables in PRIMOS must
   begin with a period, which we'll prepend */

char *gvget();
char *getenv(name)
char *name;
    {
    char temp[32];
    sprintf(temp,".%s",name);
    return(gvget(temp));
    }

/* Set RW lock to NONE for specified file */

setrwlock(fname)
char *fname;
    {
    char *strchr(),*strrchr();
    fortran void at$(),satr$$(),gpath$();
    struct cv cv,curr;
    short code;
    register char *p;

    if (p = strrchr(fname,'>'))  {

         /* Hard case, file is specified with a pathname.  First,
            we must get pathname of current attach point */

         gpath$((short)k$cura,(short)0,curr.string,(short)256,curr.length,code);
         if (code) return(code);

         /* Now copy directory path (part to left of '>') into char
            varying and attach to specified directory.  If there is
            only one '>' in the pathname, we must attach to MFD */

         if (strchr(fname,'>') == p) {
              *p = 0;
              sprintf(cv.string,"%s>MFD",fname);
              *p = '>';
              cv.length = strlen(cv.string);
              }
         else {
              cv.length = p-fname;
              strncpy(cv.string,fname,cv.length);
              }

         at$((short)k$setc,cv,code);
         if (code) return(code);

         /* Now we can use satr$$, which only deals with entry names */

         p++;
         satr$$((short)k$rwlk,(char [])p,(short)strlen(p),(short)3,code);
         if (code) return(code);

         /* How attach back to where we where */

         at$((short)k$setc,curr,code);
         }
    else

         /* Easy case, path name is just a file in current directory */

         satr$$((short)k$rwlk,(char [])fname,(short)strlen(fname),
          (short)3,code);

    return(code);
    }

/* Run specified cmd with args as background process */

run(cmd,argv)
char *cmd, **argv;
    {
    char args[256],cpl[32];
    short severity,code,pid;
    int status;
    register char *p;
    register int i;

    /* Append .CPL */

    sprintf(cpl,"%s.CPL",cmd);

    /* Concatenate args with blanks in between */

    p = args;
    *p = 0;

    for(i = 1; argv[i]; i++) p += sprintf(p,"%s ",argv[i]);

    phntm$((short)1,cpl,(short)strlen(cpl),(short)0,pid,code,
     args,(short)(strlen(args)-1));

    return((errno = code) ? 0 : pid);
    }

/* Generate background process to monitor system for specified
   amount of time without output going to specified path */

monitor_system(mins,path,append)
int mins,append;
char *path;
    {
    struct cv cv;
    short severity,code;

    sprintf(cv.string,"ph iostat %d %s %s",60*mins,path,append ? "-c" : "''");
    cv.length = strlen(cv.string);
    cp$(cv,code,severity);
    return(code);
    }

/* Determine CPU and IO times for caller */

fortran timdat();

struct {
    char      date[6];
    short     real_mins,real_secs,real_ticks;
    short     cptime[2];
    short     iotime[2];
    short     ticks_per_sec;
    } timdat_data;

cptimes(cp,ip)
double *cp;    /* where to put CPU time */
double *ip;    /* where to put IO time */
    {
    double t;

    timdat(timdat_data,(short)11);

    t = 1.0/(double)timdat_data.ticks_per_sec;
    *cp = (double)timdat_data.cptime[0] + t*(double)timdat_data.cptime[1];
    *ip = (double)timdat_data.iotime[0] + t*(double)timdat_data.iotime[1];
    }

/* Don't know how to determine time used by processes spawned by us
   (too lazy to look it up) so ignore for now */

fullcptimes(cp,ip)
double *cp,*ip;
    {
    cptimes(cp,ip);
    }

/* Return time of day as seconds since midnight with high resolution */

double time_of_day()
    {
    register long secs;
    double fracsecs,t;

    timdat(timdat_data,(short)11);

    secs = timdat_data.real_mins * 60 + timdat_data.real_secs;
    t = 1.0 / (double)timdat_data.ticks_per_sec;
    fracsecs = (double)timdat_data.real_ticks * t;
    return((double)(secs)+fracsecs);
    }

/* PRIMOS equivalent of UNIX kill */

kill(pid,signal)
int pid,signal;
    {
    short code;

    logo$$((short)1,(short)pid,(short)0,(short)0,0,code);

    return((errno = code) ? -1 : 0);
    }

/* PRIMOS equivalent of UNIX chmod.  Note: mode value must be 0666 */

chmod(fname,mode)
char *fname;
int mode;
    {
    if (mode != 0666) {
         errno = 0;
         ERRS("Can't handle mode %o",mode);
         }

    if (setmod(fname,"",7) < 0)
         ERRS("Could not set mode of %s",fname);
    }

/* Determine file size and mode.  Returned mode is 0600 if PRIMOS
   Use bit is off and 0666 if Use bit is on. A FILE_IN_USE error
   from PRIMOS means that creator is still using file, so 0600 is
   returned in this case */

#define Use_bit         04

getinfo(path,mp,sp)
char *path;
int *mp,*sp;
    {
    register int n;

    if ((n = fsize(path)) < 0) {
         if (errno != e$fius) return(-1);
         errno = 0;
         *sp = 0;
         *mp = 0600;
         }

    else {
         *sp = n;

         if ((n = getmod(path,"")) < 0) return(-1);

         *mp = n & Use_bit ? 0666 : 0600;
         }

    return(0);
    }

char *getcwd()
    {
    return("*");
    }
