/* i386locks.h
/* Copyright (c) 1988, Prime Computer, Inc., Natick, MA 01760
/*                     All Rights Reserved
*/

/* Locking macros for 386.  Code in this include file is derived from
   locking code used by Sequent Computer Systems, Inc. */

#define L_UNLOCKED      0
#define L_LOCKED        1

/*
 * Was a conditional lock request granted (L_SUCCESS) or denied (L_FAILED)
 */
#define L_FAILED        0
#define L_SUCCESS       1

/*
 * LOCK(), UNLOCK(), CLOCK(), and INIT_LOCK functions.
 * These are implemented as "asm functions", and will produce very good code.
 *
 * Need to use the -i flag with the optimizer when these inline functions are
 * used. (cc -O -i).   This is to prevent the optimizer from removing
 * "redundant" moves (which aren't really redundant in this case)
 * in the spin loops.
 */

asm void LOCK(laddr)
{
%reg laddr; lab loop, spin, done;
/PEEPOFF
loop:   movb    $L_LOCKED, %al
        xchgb   %al, (laddr)            /* an atomic "test and set" */
        cmpb    $L_UNLOCKED, %al
        je      done                    /* if equal, we got the lock */
spin:   cmpb    $L_UNLOCKED, (laddr)    /* spin until unlocked */
        je      loop                    /* then, try get lock again */
        jmp     spin
done:
/PEEPON
%mem laddr; lab loop, spin, done;
/PEEPOFF
loop:   movb    $L_LOCKED, %al
        movl    laddr, %ecx
        xchgb   %al, (%ecx)             /* an atomic "test and set" */
        cmpb    $L_UNLOCKED, %al
        je      done                    /* if equal, we got the lock */
spin:   cmpb    $L_UNLOCKED, (%ecx)     /* spin until unlocked */
        je      loop                    /* then, try get lock again */
        jmp     spin
done:
/PEEPON
}

asm void UNLOCK(laddr)
{
%reg laddr;
        movb    $L_UNLOCKED, %al
        xchgb   %al, (laddr)            /* clear lock, "atomically" */
%mem laddr;
        movb    $L_UNLOCKED, %al
        movl    laddr, %ecx
        xchgb   %al, (%ecx)             /* clear lock, "atomically" */
}

asm CLOCK(laddr)
{
%reg laddr; lab done;
        movb    $L_LOCKED, %dl
        movl    $L_SUCCESS, %eax
        xchgb   %dl, (laddr)            /* test and set lock */
        cmpb    $L_UNLOCKED, %dl        /* if we got it, return success */
        je      done
        movl    $L_FAILED, %eax         /* else return failure */
done:
%mem laddr; lab done;
        movb    $L_LOCKED, %dl
        movl    laddr, %ecx
        movl    $L_SUCCESS, %eax
        xchgb   %dl, (%ecx)             /* test and set lock */
        cmpb    $L_UNLOCKED, %dl        /* if we got it, return success */
        je      done
        movl    $L_FAILED, %eax         /* else return failure */
done:
}

#define INIT_LOCK(lp) UNLOCK(lp)
