/* ioincl.h
/* Copyright (c) 1988, Prime Computer, Inc., Natick, MA 01760
/*                     All Rights Reserved
*/

/* Common header file for IO benchmark

   Initial Version: 10 August 1988 by Barry Wolman

          Modified: 27 September 1988 by Barry Wolman
              1. Add fields to results for tracking response times

          Modified:  2 October 1988 by Barry Wolman
              1. Add calibrate field to control

          Modified: 15 February 1989 by Tom Olson
              1. Add n_reads & n_writes parameters
              2. Add tran_sum_time, tran_sumsq_time, tran_max_time

	  Modified: 25 May 1989 by Barry Wolman
              1. Add variants

          Modified: 20 June 1990 by Barry Wolman
              1. Keep UNIX user & system times separate
              2. Add fields for counted runs
              3. Add checkpoining

          Modified: 16 July 1990 by Barry Wolman
              1. Add mount/umount control
              2. Add fields for run validation

          Modified: 19 July 1990 by Barry Wolman
              1. Removed setting of UNIX flag

          Modified: 24 July 1990 by Barry Wolman
              1. Added ERRi macros & errline definition
              2. Added logrec
              3. Removed last_ fields from results

          Modified: 6 August 1990 by Barry Wolman
              1. Change logrec l_record to long (oops!)
              2. Add ERRSSS andMSGSSS macros
              3. clear errno as part of MSG* macros
*/

#if UNIX
#define fexists(x) (access((x),0) == 0)
extern int errno;
#endif

#define MAX_FILES  20
#define MAX_USERS  1024
#define MAX_RECLEN 8192      /* Should be 16 or greater */

/* The following value is chosen so that when files are initialized
   on UNIX systems there are no all zero pages */

#define INITRECLEN 512

struct rec {
    long      counter;
    long      time;
    int       user;
    char      junk[MAX_RECLEN];
    };

struct logrec {
    short     l_file;
    long      l_record,l_counter,l_time;
    };

#define NUMBER_OF_CHECKPOINTS 10

struct checkpoint {
    long      delta,counter;
    double    usert,syst;
    };

struct results {
    long      userno,counter,start,finish;
    int       n_files,n_recs,n_users,locks_per_file,
               reclen,duration,lock_sleep_time,param;
    int       opens[MAX_FILES],closes[MAX_FILES],uses[MAX_FILES];
    long      freads[MAX_FILES],fwrites[MAX_FILES];
    double    sum_time[MAX_FILES],sumsq_time[MAX_FILES],worst_time[MAX_FILES];
    double    tran_sum_time,tran_sumsq_time,tran_max_time;
    int       worst_rec[MAX_FILES];
    double    usert,syst,work_cp;
    long      reads,writes;
    long      resp_dist[26];
    struct    checkpoint checkpoint[NUMBER_OF_CHECKPOINTS+1];
    };

struct control {
    int       runid,runtype;
    int       n_files,n_recs,n_users,locks_per_file,
               reclen,duration,lock_sleep_time,param,mounts;
    int       start_delay,end_delay,filelen;
    int       n_reads,n_writes,trans;
    int       calibrate;
    int       variant;
    char      *path[MAX_FILES],*dir[MAX_FILES],*device[MAX_FILES];
    };

#define LOCKING_VARIANT 1

static char errline[128];

/* ERR, ERRS, ERRSS, and ERRSSS macros print an error message on stderr
   (via gripe) and call exit (via gripe). */

#define ERR(msg) gripe(msg,1)

/* Following macros use comma to separate the sprintf and gripe
   calls so that ERRS and ERRSS can be treated as single "statement", e.g.
        if (...) ERRS(...,...);
   If semicolon were used as the separator, braces would be needed, e.g.
        if (...) { ERRS(...,...); }
*/

#define ERRS(msg,str) sprintf(errline,msg,str) , gripe(errline,1)
#define ERRSS(msg,s1,s2) sprintf(errline,msg,s1,s2) , gripe(errline,1)
#define ERRSSS(msg,s1,s2,s3) sprintf(errline,msg,s1,s2,s3) , gripe(errline,1)

/* MSG, MSGS, MSGSS, and MSGSSS macros print an error message on stderr
   (via gripe) and continue execution */

#define MSG(msg) gripe(msg,(errno = 0))
#define MSGS(msg,str) sprintf(errline,msg,str) , gripe(errline,(errno = 0))
#define MSGSS(msg,s1,s2) sprintf(errline,msg,s1,s2) , gripe(errline,(errno = 0))
#define MSGSSS(msg,s1,s2,s3) sprintf(errline,msg,s1,s2,s3) , gripe(errline,(errno = 0))
