/* iorandom.c
/* Copyright (c) 1990, Prime Computer, Inc., Natick, MA 01760
/*                     All Rights Reserved
*/

/* Random number module for IOBENCH.

   random() delivers a random number uniformly distributed in
   the range 0.0-1.0

   getseed() returns current seed

   setseed(val) sets seed to specified value

   setrandom(n) skips the next "n" numbers

   Initial Version: 10 June 1990 by Barry Wolman

          Modified: 8 October 1990 by Barry Wolman
              1. Use SPEC standard random number generator

          Modified: 6 November 1990 by Barry Wolman
              1. Add getseed and setseed
*/


/* See "Random Number Generators: Good Ones Are Hard To Find", */
/*     Park & Miller, CACM 31#10 October 1988 pages 1192-1201. */
/***********************************************************/
/* THIS IMPLEMENTATION REQUIRES AT LEAST 32 BIT INTEGERS ! */
/***********************************************************/

#define _A_MULTIPLIER  16807L
#define _M_MODULUS     2147483647L /* (2**31)-1 */
#define _Q_QUOTIENT    127773L     /* 2147483647 / 16807 */
#define _R_REMAINDER   2836L       /* 2147483647 % 16807 */

static long seedi = 1973272912L;

long getseed()
    {
    return(seedi);
    }

long setseed(newseed)
long newseed;
    {
    register long oldseed;

    oldseed = seedi;
    seedi = newseed;
    return(oldseed);
    }

double random()
    {
    long lo;
    long hi;
    long test;

    hi = seedi / _Q_QUOTIENT;
    lo = seedi % _Q_QUOTIENT;
    test = _A_MULTIPLIER * lo - _R_REMAINDER * hi;

    if (test > 0) {
         seedi = test;
         }
    else {
         seedi = test + _M_MODULUS;
         }

    return ( (double) seedi / _M_MODULUS);
    }

/* Skip the next n random numbers */

void setrandom (n)
register int n;
    {
    while(--n >= 0) random();
    }
