/* ioserver.c
/* Copyright (c) 1989, Prime Computer, Inc., Natick, MA 01760
/*                     All Rights Reserved
*/

/* ioserver for IOBENCH

   The following message types are handled:

         INIT_SERVER_MSG     Initialize
         INIT_LOCK_MSG       Initialize locking
         LOCK_MSG            Lock a lock
         UNLOCK_MSG          Unlock a lock
         LOCK_INFO_MSG       Return lock statistics
         LOCK_ERRORS_MSG     Return lock errors
         TERM_LOCK_MSG       Terminate locking
         TERM_SERVER_MSG     Terminate

   Initial Version:     23 May 1989 by Barry Wolman

          Modified:     18 May 1990 by Barry Wolman
              1. Add hit information to LOCK_INFO_MSG

          Modified:     20 June 1990 by Barry Wolman
              1. Use cptimes instead of systimes, rename variables

          Modified:     24 July 1990 by Barry Wolman
              1. Use ERR macro instead of gripe

*/

#include <stdio.h>
#include "ioincl.h"
#include "iolocks.h"    /* Defines lock_t and locking macros */
#include "ioserver.h"

int n_users,n_files,locks_per_file,server_locking;
double usert1,usert2,syst1,syst2;

main(argc,argv)
char **argv;
    {
    extern cleanup();
    int i,working ;
    FILE *fp;
    struct msgtime *p;

    cptimes(&usert1,&syst1);

#if UNIX
    for (i = 0; i < 20; i++) signal(i,cleanup);
#endif

    /* To avoid a race condition when ioserver is started, IOBENCH
       creates the message queue, we just attach to it */

    if ((rmsgid = msgget(INKEY,0777)) < 0)
     ERR("msgget failed for input queue");

    if ((smsgid = msgget(OUTKEY,0777)) < 0)
     ERR("msgget failed for output queue");

    user = -1;
    mypid = getpid();

    /* Now loop waiting for messages */

    for(i=working=1;working;i++) {

         if (RECEIVE(rmsgid,rmsg,0) < 0) ERR("loop msgrcv failed");

#ifdef SERVERDEBUG
         printf("server: message %d is type %d from %d, items = %d, %d, %d\n",i,
          rmsg.mtype,rmsg.mitem[0],rmsg.mitem[1],rmsg.mitem[2],rmsg.mitem[3]);
         fflush(stdout);
#endif
         /* Assume success */

         smsg.mitem[0] = 0;

         switch(rmsg.mtype) {

         case INIT_SERVER_MSG:
              break;

         case INIT_LOCK_MSG:
              initlocks(rmsg.mitem[1],rmsg.mitem[2],rmsg.mitem[3],0);
              break;

         case LOCK_MSG:
              smsg.mitem[0] = server_lockrec(rmsg.mitem[1],rmsg.mitem[2],
               rmsg.mitem[3]);
              break;

         case UNLOCK_MSG:
              smsg.mitem[0] = server_unlockrec(rmsg.mitem[1],rmsg.mitem[2],
               rmsg.mitem[3]);
              break;

         case LOCK_INFO_MSG:
              lockinfo(rmsg.mitem[1],&smsg.mitem[1],&smsg.mitem[2],
               &smsg.mitem[3],&smsg.mitem[4]);
              break;

         case LOCK_ERRORS_MSG:
              smsg.mitem[1] = lockerrors(rmsg.mitem[1],rmsg.mitem[2]);
              break;

         case TERM_LOCK_MSG:
              termlocks();
              break;

         case SERVER_TIMES_MSG:

              /* Pass back CP & IO time used in server */

              cptimes(&usert2,&syst2);
              p = (struct msgtime *)&smsg;
              p -> usert = usert2 - usert1;
              p -> syst = syst2 - syst1;
              break;

         case TERM_SERVER_MSG:
              working = 0;                  /* Stop after sending reply */

              }

         /* Send reply to requesting process, use requestor's process
            id as reply message type */

         smsg.mtype = rmsg.mitem[0];

#ifdef SERVERDEBUG
         printf("server: replying %d to %d\n",smsg.mitem[0],smsg.mtype);
         fflush(stdout);
#endif

         if (SEND(smsgid,smsg) < 0) ERR("loop msgsnd failed");

         /* Wait for next message */

         }

#ifdef SERVERDEBUG
    printf("server: exiting\n");
#endif

    exit(0);
    }

cleanup()
    {

    /* Since IOBENCH created the message queues, we'll let IOBENCH destroy them */

    exit(0);
    }
