/* ismounted.c
/* Copyright (c) 1990, Prime Computer, Inc., Natick, MA 01760
/*                     All Rights Reserved
*/

/* Subroutine ismounted(device) tries to determine if the specified
   device is currently mounted.  ismounted returns

         -1   on error
          0   if not mounted
          1   if mounted

   Unfortunately, BSD and System V variants of UNIX provide different
   mechanisms for determining if a device is mounted, so -DBSD or -DSYSV
   must be specified to compile this program.  If neither can be used,
   specify -DNEITHER to build dummy versions of routines in this module.

   Subroutine getdevice(path) tries to determine the device on which
   the specified path is mounted.

   Initial Version: 25 October 1990 by Barry Wolman
*/

#include <stdio.h>

#ifdef BSD
#include <mntent.h>

char *index();

FILE *setmntet();
struct mntent *getmntent();
int endmntent();

int ismounted(device)
char *device;
    {
    register FILE *fp;
    register struct mntent *mp;
    int result;

    if ((fp = setmntent("/etc/mtab","r")) == NULL) return(-1);

    result = 0;
    while(mp = getmntent(fp))
         if (strcmp(mp -> mnt_fsname,device) == 0) result = 1;

    endmntent(fp);
    return(result);
    }

char *getdevice(path)
char *path;
    {
    register FILE *fp;
    register struct mntent *mp;
    register char *p;
    char temp[32],*result;
    static char device[32];

    if (path == NULL) return(NULL);

    if ((fp = setmntent("/etc/mtab","r")) == NULL) return(NULL);

    strcpy(temp,path);
    if (p = index(temp+1,'/')) *p = 0;

    result = NULL;
    while(mp = getmntent(fp))
         if (strcmp(mp -> mnt_dir,temp) == 0) {
              strcpy(device,mp -> mnt_fsname);
              result = device;
              }


    endmntent(fp);
    return(result);
    }
#else

#ifdef SYSV
#include <sys/types.h>
#include <mnttab.h>

char *strchr();

int ismounted(device)
char *device;
    {
    register FILE *fp;
    struct mnttab mnttab;
    int result;

    if ((fp = fopen("/etc/mnttab","r")) == NULL) return(-1);

    result = 0;
    while(fread(&mnttab, sizeof(mnttab), 1, fp) == 1)
         if (strcmp(mnttab.mt_dev,device) == 0) result = 1;

    fclose(fp);
    return(result);
    }

char* getdevice(path)
char *path;
    {
    register FILE *fp;
    struct mnttab mnttab;
    register char *p;
    char temp[32],*result;
    static char device[32];

    if (path == NULL) return(NULL);

    if ((fp = fopen("/etc/mnttab","r")) == NULL) return(NULL);

    strcpy(temp,path);
    if (p = strchr(temp+1,'/')) *p = 0;

    result = NULL;
    while(fread(&mnttab, sizeof(mnttab), 1, fp) == 1)
         if (strcmp(mnttab.mt_filsys,temp) == 0) {
              strcpy(device,mnttab.mt_dev);
              result = device;
              }

    fclose(fp);
    return(result);
    }
#else
#ifdef NEITHER
int ismounted(device)
char *device;
    {
    return(1);
    }

char *getdevice(path)
char *path;
    {
    return(NULL);
    }
#else
extern FILE *of[];
int ismounted(device)
char *device;
    {
    static int first_time = 1;
    register int z;

    if (first_time) {
         for(z=0;of[z];z++) fprintf(of[z],
"\nWarning: You did not define either BSD or SYSV flags when you made\n\
IOBENCH, so it is unable to determine what file systems are currently\n\
mounted.  You may see an error message from an attempt to unmount an\n\
already unmounted file system;  you can ignore such a message.  In order\n\
for IOBENCH to check if a file system is mounted, add either -DBSD or\n\
-DSYSV (as appropriate) to the EXTRA_CFLAGS in your Makefile wrapper and\n\
re-make IOBENCH.  If neither -DBSD or -DSYSV is appropriate, add -DNEITHER\n\
to the EXTRA_CFLAGS in your Makefile wrapper and re-make IOBENCH.\n\n");
         first_time = 0;
         }

    return(1);
    }

char *getdevice(path)
char *path;
    {
    return(NULL);
    }

#endif
#endif
#endif
